@@ -23,7 +23,17 @@ class Auth:
     def __init__ (self):
         self._need_authenticate = None
         self._permission_map = {}
+        self._auth_requires = {}
     
+    def set_auth_flag (self, f, required):
+        func_id = self.get_func_id (f)
+        if func_id not in self._auth_requires:
+            self._auth_requires [func_id] = set ()        
+        self._auth_requires [func_id].add (required)
+
+    def get_auth_flags (self, func_id):
+        return list (self._auth_requires.get (func_id, []))
+
     # high level API with skitai----------------------------------------------    
     def get_www_authenticate (self, authenticate, error = None):
         if authenticate == "bearer":
@@ -162,6 +172,7 @@ class Auth:
     def authorization_required (self, authenticate):
         def decorator (f):
             self.save_function_spec (f)
+            self.set_auth_flag (f, ('authorization', authenticate))
             authenticate_ = authenticate.lower ()
             assert authenticate_ in self.AUTH_TYPES            
             self._need_authenticate = (f.__name__, authenticate_)
@@ -175,6 +186,7 @@ class Auth:
     
     def login_required (self, f):
         self.save_function_spec (f)
+        self.set_auth_flag (f, ('login', None))
         @wraps(f)
         def wrapper (was, *args, **kwargs):
             _funcs = self._decos.get ("login_handler")
@@ -192,6 +204,7 @@ class Auth:
     
     def staff_member_required (self, f):
         self.save_function_spec (f)
+        self.set_auth_flag (f, 'staff')
         @wraps(f)
         def wrapper (was, *args, **kwargs):
             _funcs = self._decos.get ("staff_member_check_handler")
@@ -210,12 +223,13 @@ class Auth:
     METHODS = {"POST", "GET", "PUT", "DELETE", "PATCH"}
     def permission_required (self, __donotusethisvariable__ = None, **kargs):
         if not __donotusethisvariable__:
-            __donotusethisvariable__ = {}
+            __donotusethisvariable__ = set ()
         else:
             __donotusethisvariable__ = set (__donotusethisvariable__)
 
         def decorator(f):
             self.save_function_spec (f)
+            self.set_auth_flag (f, ('permission', tuple (__donotusethisvariable__)))
             methods = {}
             specified = {}
             for k, v in kargs.items ():
