@@ -33,7 +33,7 @@ class Decorators:
         if func_id not in self._function_specs or func_id not in self._current_function_specs:
             # save origin spec
             self._function_specs [func_id] = inspect.getfullargspec (func)
-            self._current_function_specs [func_id] = None
+            self._current_function_specs [func_id] = None            
     
     def get_function_spec (self, func):
         # called by websocet_handler 
@@ -117,12 +117,18 @@ class Decorators:
                     return 'parameter {} required'.format (each)                                                
         if ints:
             for each in ints:
-                try: int (params [each])
+                try: 
+                    params [each] = int (params [each])
+                except KeyError:
+                    pass    
                 except ValueError:
                     return 'parameter {} should be integer'.format (each)                                   
         if floats:
             for each in floats:
-                try: float (params [each])
+                try: 
+                    params [each] = float (params [each])
+                except KeyError:
+                    pass    
                 except ValueError:
                     return 'parameter {} should be float'.format (each)                    
         if emails:
@@ -184,7 +190,7 @@ class Decorators:
     def get_parameter_requirements (self, func_id):
         return self._parameter_caches.get (func_id, {})        
 
-    def test_params (self, scope, required = None, ints = None, floats = None, uuids = None, emails = None, **kargs):
+    def require (self, scope, required = None, ints = None, floats = None, uuids = None, emails = None, **kargs):
         def decorator(f):
             self.save_function_spec (f)
             func_id = self.get_func_id (f)
@@ -193,19 +199,33 @@ class Decorators:
             self._parameter_caches [func_id][scope] = (required, ints, floats, uuids)
             @wraps(f)
             def wrapper (was, *args, **kwargs):
+                scope_ = scope
                 if scope in ("FORM", "JSON"):
                     if was.request.method not in {"POST", "PUT", "PATCH"}:
                         return f (was, *args, **kwargs)
                     if scope == "JSON" and not was.request.get_header ("content-type", '').startswith ("application/json"):
                         return f (was, *args, **kwargs)
-                more_info = self._validate_param (getattr (was.request, scope), required, ints, floats, emails, uuids, **kargs)
+
+                elif scope not in ("URL", "ARGS"):
+                    if was.request.method != scope:
+                        return f (was, *args, **kwargs)
+                    if scope in {"GET", "DELETE"}:                    
+                        scope_ = "URL"
+                    elif scope in {"POST", "PUT", "PATCH"}:
+                        if was.request.get_header ("content-type", '').startswith ("application/json"):
+                            scope_ = "JSON"
+                        else:
+                            scope_ = "FORM"
+                    else:
+                        return f (was, *args, **kwargs)
+
+                more_info = self._validate_param (getattr (was.request, scope_), required, ints, floats, emails, uuids, **kargs)
                 if more_info:
-                    return was.response.adaptive_error ("400 Bad Request", 'missing or bad parameterin {}'.format (scope), 40050, more_info) 
+                    return was.response.adaptive_error ("400 Bad Request", 'missing or bad parameterin {}'.format (scope_), 40050, more_info) 
                 return f (was, *args, **kwargs)
             return wrapper
         return decorator
-    params_required = test_params
-    parameters_required = test_params
+    parameters_required = params_required = test_params = require
     
     # Automation ------------------------------------------------------    
     def run_before (self, *funcs):
