@@ -212,8 +212,8 @@ class WAS (wsgiappservice.WAS):
         return self.response.File (*args, **karg)
     
     def proxypass (self, alias, path, timeout = 3):
-        def respond (was, rss):
-            resp = rss [0]
+        def respond (was, rs):
+            resp = rs.dispatch ()
             resp.reraise () 
             [was.response.update (k, v) for k, v in resp.headers.items () if k not in ("Content-Length",)]
             return was.response (
@@ -222,5 +222,5 @@ class WAS (wsgiappservice.WAS):
             )
         headers = dict ([(k, v) for k, v in self.request.headers.items ()])
         headers ["Accept"] = "*/*"
-        return self.Futures ([self.get ("{}/{}".format (alias, path), headers = headers)], timeout = timeout).then (respond)
-    
\ No newline at end of file
+        return self.get ("{}/{}".format (alias, path), headers = headers, timeout = timeout).then (respond)
+    ProxyPass = proxypass
